# -*- coding: utf-8 -*-

from typing import Optional

from PyQt5.QtCore import QPoint
from PyQt5.QtWidgets import QFrame, QWidget
from PyQt5.QtGui import QMouseEvent

from fly_csp_plugins.signature_mark_widget.mark import MarkLabel


class MarkWidget(QFrame):
    def __init__(self, parent: QWidget) -> None:
        super().__init__(parent)
        self.__mark_label = MarkLabel(parent=self)
        self.__last_mouse_move_position = None  # type: Optional[QPoint]
        self.__setup()

    @property
    def mark_label(self) -> MarkLabel:
        return self.__mark_label

    def scale(self, factor) -> None:
        self.__mark_label.scale_factor = factor * 100
        self.__mark_label.update_content()
        self.resize(self.__mark_label.width(), self.__mark_label.height())

    def mousePressEvent(self, event: QMouseEvent) -> None:
        self.__last_mouse_move_position = event.globalPos()

        super().mousePressEvent(event)

    def mouseReleaseEvent(self, event: QMouseEvent) -> None:
        self.__last_mouse_move_position = None

        super().mouseReleaseEvent(event)

    def mouseMoveEvent(self, event: QMouseEvent) -> None:
        if self.__last_mouse_move_position is None:
            self.__last_mouse_move_position = event.globalPos()

        bias = event.globalPos() - self.__last_mouse_move_position

        geometry = self.geometry()
        geometry.moveTo(geometry.topLeft() + bias)
        if self.parent().rect().contains(geometry):
            self.move(self.pos() + bias)

        self.__last_mouse_move_position = event.globalPos()

    def __setup(self) -> None:
        self.__mark_label.setParent(self)
