# -*- coding: utf-8 -*-

from typing import Optional

from PyQt5.QtCore import QPoint, pyqtSignal
from PyQt5.QtWidgets import QScrollArea, QWidget
from PyQt5.QtGui import QWheelEvent, QMouseEvent


class ScrollArea(QScrollArea):
    wheel_up_triggered = pyqtSignal()
    wheel_down_triggered = pyqtSignal()

    def __init__(self, parent: QWidget) -> None:
        super().__init__(parent)
        self.__last_mouse_move_position = None  # type: Optional[QPoint]

    def wheelEvent(self, event: QWheelEvent) -> None:
        if event.angleDelta().y() > 0:
            self.wheel_up_triggered.emit()
        else:
            self.wheel_down_triggered.emit()

    def mousePressEvent(self, event: QMouseEvent) -> None:
        self.__last_mouse_move_position = event.pos()

        super().mousePressEvent(event)

    def mouseReleaseEvent(self, event: QMouseEvent) -> None:
        self.__last_mouse_move_position = None

        super().mouseReleaseEvent(event)

    def mouseMoveEvent(self, event: QMouseEvent) -> None:
        if self.__last_mouse_move_position is None:
            self.__last_mouse_move_position = event.pos()

        bias = self.__last_mouse_move_position - event.pos()
        self.horizontalScrollBar().setValue(self.horizontalScrollBar().value() + bias.x())
        self.verticalScrollBar().setValue(self.verticalScrollBar().value() + bias.y())

        self.__last_mouse_move_position = event.pos()
