# -*- coding: utf-8 -*-

import os

from typing import Optional

from tempfile import NamedTemporaryFile
from shutil import copyfile
from PyPDF2 import PdfFileReader, PdfFileWriter

from PyQt5.QtCore import QThread, QObject


class WatermarkCreationThread(QThread):
    def __init__(self,
                 source_path: str,
                 watermark_path: str,
                 preview_path: str,
                 watermark_page_index: int,
                 parent: QObject = None) -> None:
        self.__source_path = source_path
        self.__watermark_path = watermark_path
        self.__preview_path = preview_path
        self.__watermark_page_index = watermark_page_index
        self.__error = None  # type: Optional[str]
        super().__init__(parent)

    @property
    def preview_path(self) -> str:
        return self.__preview_path

    @property
    def error(self) -> Optional[str]:
        return self.__error

    def run(self) -> None:
        self.__error = None
        try:
            self.__create_watermark()
        except Exception as exception:
            self.__error = str(exception)

    def __create_watermark(self) -> None:
        with open(self.__source_path, 'rb') as source:
            s_pdf = PdfFileReader(source)
            s_page = s_pdf.getPage(self.__watermark_page_index)

            with open(self.__watermark_path, 'rb') as watermark:
                w_pdf = PdfFileReader(watermark)
                s_page.mergePage(w_pdf.getPage(0))

                writer = PdfFileWriter()
                for page_index in range(s_pdf.getNumPages()):
                    if page_index == self.__watermark_page_index:
                        writer.addPage(s_page)
                    else:
                        writer.addPage(s_pdf.getPage(page_index))

                preview = NamedTemporaryFile('wb', dir=os.path.dirname(self.__preview_path))
                writer.write(preview)
                preview.flush()
                copyfile(preview.name, self.__preview_path)
                preview.close()
